// Native_Polymorphism.cpp : Defines the entry point for the console application.
/*
________________________________________

P O L Y M O R P H I S M   D E M O
Ron Kessler
Created 2/25/2017

This demo combines inheritance with polymorphic behavior.
I define a base class named Shapes with an area method
marked as virtual. That means any child/derived classes
MUST inherit and and override the area method. 

One derived class is the rectangle class and the other is the triangle
class. They also have an area method but with different formulas
based on the shape.

We can then call the area method in Shapes the compiler, at run-time,
will know which object we are wanting the area from and choose the correct 
area method in one of the derived classes.

This is analagous to overloaded methods. BUT, here we are talking about different
types of object, not methods.

This uses late-binding. The compiler only knows what object to work with at run-time.
Early binding refers to the compiler figuring things out at compile time.

Updated 3/12/2017  Removed all const references to simplify this demo.
*/

#pragma run once



#include "stdafx.h"
#include <iostream>

#include "ARectangle.h"
#include "Triangle.h"

using namespace std;

int compute_area (Shapes&);		
//int compute_area (const Shapes&);		//using const to assure we do not change original values

int main()
{
	//---add a title
	cout << "WORKING WITH INHERITANCE & POLYMORPHISM.\n\n";
	cout << "We will compute the area of a 20 X 20 living room.\n";
	cout << "Then we will compute the area of a triangle-shaped patio of 25 X 30.\n\n\n";

	//---create new living room object and initialize it with these dimensions
	ARectangle livingRoom{ 20, 20 };
	cout << "The area of the Living Room is " << compute_area(livingRoom) << " square feet.\n\n\n";
	
	
	
	//---create new triangle-shaped patio object and initialize it with these dimensions
	Triangle patio{ 25, 30 };
	cout << "The area of your triangle-shaped patio is " << compute_area (patio) << " square feet.\n\n";


    return 0;
}


//my helper function
//int compute_area (const Shapes& my_shape)	//using const

int compute_area (Shapes& my_shape)		//we pass in a reference to Shapes Class
	{
		return my_shape.area ();							//it calls triangle.area when we pass in a triangle object
																		//it calls rectangle.area when we pass in a rectangle object
	}

